from django.conf.urls.defaults import *


urlpatterns = patterns("magneto.views",
    url(r"^$", "detail", dict(name="index"), name="magneto.detail.index"),
    url(r"^(.+?)/?$", "detail", name="magneto.detail"),
)



from difflib import unified_diff
import os

from django import forms
from django.conf import settings
from django.contrib import admin
from django.utils.safestring import mark_safe

from reversion.admin import VersionAdmin

from models import Group, Template


MEDIA_PREFIX = getattr(settings, "MAGNETO_MEDIA_DIRECTORY",
    os.path.join(settings.MEDIA_URL, "magneto/"))


class CodeMirrorTextArea(forms.Textarea):

    class Media:
        css = dict(screen=[os.path.join(MEDIA_PREFIX,
            "codemirror/css/editor.css")])
        js = [os.path.join(MEDIA_PREFIX, "codemirror/js/codemirror.js")]

    def render(self, name, value, attrs=None):
        result = []
        result.append(super(CodeMirrorTextArea,
            self).render(name, value, attrs))

        result.append(u"""<p>View the Django <a href="http://docs.djangoproject.com/en/dev/ref/templates/builtins/">built-in tags and filters</a>.</p>""")

        result.append(u"""
<script type="text/javascript">
  var editor = CodeMirror.fromTextArea('id_%(name)s', {
    path: "%(media_url)scodemirror/js/",
    parserfile: "parsedjango.js",
    stylesheet: "%(media_url)scodemirror/css/django.css",
    continuousScanning: 500,
    height: "40.2em",
    tabMode: "shift",
    indentUnit: 2,
    lineNumbers: true
  });
</script>
""" % dict(media_url=MEDIA_PREFIX, name=name))
        return mark_safe(u"".join(result))


class GroupAdmin(admin.ModelAdmin):
    pass
admin.site.register(Group, GroupAdmin)


class TemplateAdminForm(forms.ModelForm):
    content = forms.CharField(
        widget=CodeMirrorTextArea(dict(rows=24))
    )

    class Meta:
        model = Template


class TemplateAdmin(VersionAdmin):
    actions = ("copy_templates", )
    date_hierarchy = "date_created"
    form = TemplateAdminForm
    list_display = ("name", "content_type", "group", "site", "visible", )
    list_filter = ("content_type", "group", "visible", "site", )
    list_per_page = 100
    object_history_template = "magneto/object_history.html"
    ordering = ("name", )
    readonly_fields = ("date_created", "date_changed",
        "last_modified_by", )
    search_fields = ("name", )

    def __init__(self, *args, **kwargs):
        self.message = "Changed content."
        super(TemplateAdmin, self).__init__(*args, **kwargs)

    def copy_templates(self, request, queryset):
        for obj in queryset:
            Template.objects.create(
                name="%s-copy" % obj.name,
                content_type=obj.content_type,
                content=obj.content,
                group=obj.group,
                site=obj.site,
                visible=obj.visible,
            )
    copy_templates.short_description = "Copy selected templates"

    def log_change(self, request, obj, message=""):
        message = message or self.message
        super(TemplateAdmin, self).log_change(request, obj, self.message)

    def save_model(self, request, obj, form, change):
        if obj.pk:
            self.message = generate_diff(
                Template.objects.get(pk=obj.pk), obj)
        obj.last_modified_by = request.user
        obj.save()
admin.site.register(Template, TemplateAdmin)


def generate_diff(old, new):
    c1 = old.content.split("\n")
    c2 = new.content.split("\n")
    return "\n".join(list(unified_diff(c1, c2)))



from datetime import datetime

from django.contrib.sites.models import Site
from django.contrib.sites.managers import CurrentSiteManager
from django.core.cache import cache
from django.db import models
from django.utils.translation import gettext_lazy as _

from magneto import cache_key


def get_current_site():
    return Site.objects.get_current().pk


class Group(models.Model):
    name = models.CharField(_("Name"), unique=True, max_length=100)

    def __unicode__(self):
        return self.name


class Template(models.Model):

    TYPE_CHOICES = (
        ("text/html", "Web Page"),
        ("text/plain", "Template"),
        ("text/css", "CSS Stylesheet"),
        ("text/javascript", "Javascript"),
        ("application/json", "JSON"),
        ("text/xml", "XML"),
    )

    name = models.CharField(_("Name"), unique=True, max_length=200)
    content_type = models.CharField(max_length=32, choices=TYPE_CHOICES,
        default="text/html")
    content = models.TextField(_("Content"), blank=True)
    group = models.ForeignKey("magneto.Group")
    site = models.ForeignKey("sites.Site", default=get_current_site)
    visible = models.BooleanField(default=False)
    date_created = models.DateTimeField( _("Date Created"),
        default=datetime.now)
    date_changed = models.DateTimeField(_('Date Changed'),
        default=datetime.now)
    last_modified_by = models.ForeignKey("auth.User",
        blank=True, null=True)

    objects = models.Manager()
    on_site = CurrentSiteManager("site")

    class Meta:
        ordering = ("name", )
        unique_together = ("name", "site")
        verbose_name = _("Template")
        verbose_name_plural = _("Templates")

    def __unicode__(self):
        return self.name
    
    def save(self, *args, **kwargs):
        self.name = self.name.strip()
        self.name = self.name.strip("/")
        self.date_changed = datetime.now()
        super(Template, self).save(*args, **kwargs)
        cache.delete(cache_key(self.name))

    @models.permalink
    def get_absolute_url(self):
        if self.name == "index":
            return ("magneto.detail.index", )
        else:
            return ("magneto.detail", [self.name])



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Group'
        db.create_table('magneto_group', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
        ))
        db.send_create_signal('magneto', ['Group'])

        # Adding model 'Template'
        db.create_table('magneto_template', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('content_type', self.gf('django.db.models.fields.CharField')(default='text/html', max_length=32)),
            ('content', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['magneto.Group'])),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(default=1, to=orm['sites.Site'])),
            ('visible', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_changed', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('last_modified_by', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], null=True, blank=True)),
        ))
        db.send_create_signal('magneto', ['Template'])

        # Adding unique constraint on 'Template', fields ['name', 'site']
        db.create_unique('magneto_template', ['name', 'site_id'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'Template', fields ['name', 'site']
        db.delete_unique('magneto_template', ['name', 'site_id'])

        # Deleting model 'Group'
        db.delete_table('magneto_group')

        # Deleting model 'Template'
        db.delete_table('magneto_template')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'magneto.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        },
        'magneto.template': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('name', 'site'),)", 'object_name': 'Template'},
            'content': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'content_type': ('django.db.models.fields.CharField', [], {'default': "'text/html'", 'max_length': '32'}),
            'date_changed': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['magneto.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_modified_by': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': "orm['sites.Site']"}),
            'visible': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['magneto']



from django.contrib.sites.models import Site
from django.http import Http404
from django.shortcuts import render_to_response
from django.template import RequestContext

from models import Template


def detail(request, name):
    """
    Render a page for a ``Template`` object that matches the current
    site and ``name`` keyword argument. If no matching ``Template``
    is found or if the ``Template`` is not marked as ``visible``,
    raise a 404.
    
    The content type of the response corresponds to the content_type
    field of the ``Template`` object.

    **Required arguments**

        ``name``
            The name of the ``Template`` object to match.

    **Context**

        ``template``
            The ``Template`` object being rendered.
    """

    try:
        template_instance = Template.objects.get(
            name=name,
            visible=True,
            site=Site.objects.get_current(),
        )
    except Template.DoesNotExist:
        raise Http404

    return render_to_response(
        template_instance.name,
        dict(template=template_instance),
        context_instance=RequestContext(request),
        # KLUDGE: Coerce the content_type field from unicode to string.
        # Otherwise, Apache and nginx seem to have random issues with
        # interpreting the content type.
        mimetype=str(template_instance.content_type),
    )



from django.contrib.sites.models import Site
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.db import connection
from django.template import TemplateDoesNotExist
from django.test import TestCase
from django.test.client import Client

from loader import CachedTemplateLoader
from models import Group, Template


class TemplateTest(TestCase):

    urls = "magneto.urls"

    def test_default_site(self):
        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="about/staff",
        )
        self.assertEqual(template.site,
            Site.objects.get(domain="example.com"))

    def test_absolute_url(self):
        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="about/staff",
        )
        self.assertEqual(template.get_absolute_url(), "/about/staff")

    def test_absolute_url_index(self):
        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="index",
        )
        self.assertEqual(template.get_absolute_url(), "/")

    def test_strip(self):
        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name=" /about/staff/   ",
        )
        self.assertEqual(template.name, "about/staff")


class CachedTemplateLoaderTest(TestCase):

    def test_loader(self):
        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="about/staff",
            content="Hello World",
        )

        self.assertEqual(
            CachedTemplateLoader().load_template_source("about/staff"),
            (u"Hello World", "about/staff"))
        self.assertEqual(template.site,
            Site.objects.get(domain="example.com"))

        self.assertEqual(cache.get("01c061d1a685bf3dbbed752c339bb579"), "Hello World")

    def test_loader_with_index_value(self):
        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="index",
            content="Hello World",
        )

        self.assertEqual(CachedTemplateLoader().load_template_source(""),
            (u"Hello World", "index"))

    def test_loader_restricted_to_current_site(self):
        cache.clear()
        s1 = Site.objects.create(domain="site1.com", name="site1")
        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="about/staff",
            content="Hello World",
            site=s1,
        )

        self.assertRaises(TemplateDoesNotExist,
            CachedTemplateLoader().load_template_source, "about/staff")

    def test_loader_caches_misses(self):
        group = Group.objects.create(name="Admin")
        template = Template.objects.create(
            group=group,
            name="about/admin",
            content='{% extends "admin/change_form.html" %}',
        )

        self.assertRaises(TemplateDoesNotExist,
            CachedTemplateLoader().load_template_source,
            "admin/change_form.html")
        self.assertEqual(cache.get("8eb1e907540975a50e9624b8766e58ea"), "miss")


class DetailViewTest(TestCase):

    urls = "magneto.urls"

    def test_detail(self):
        client = Client()

        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="about/staff",
            content="Hello World",
            visible=True,
        )
        response = client.get(reverse("magneto.detail",
            args=["about/staff"]))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response._headers["content-type"][1], "text/html")

    def test_detail_index(self):
        client = Client()

        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="index",
            content="Hello World",
            visible=True,
        )
        response = client.get(reverse("magneto.detail.index"))
        self.assertEqual(response.status_code, 200)

    def test_detail_404_if_not_visible(self):
        client = Client()

        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="about/staff",
            content="Hello World",
            visible=False,
        )
        response = client.get(reverse("magneto.detail",
            args=["about/staff"]))
        self.assertEqual(response.status_code, 404)

    def test_detail_content_type(self):
        client = Client()

        group = Group.objects.create(name="Home")
        template = Template.objects.create(
            group=group,
            name="css/main.css",
            content="body { color: #ccc }",
            content_type="text/css",
            visible=True,
        )
        response = client.get(reverse("magneto.detail",
            args=["css/main.css"]))
        self.assertEqual(response.status_code, 200)

        self.assertEqual(response._headers["content-type"][1], "text/css")

        # Make sure content type is sent as a string and not unicode
        # Unicode values seem to confuse either Apache or nginx
        self.assertEqual(type(response._headers["content-type"][1]),
            type(""))



import hashlib

from django.core.cache import cache
from django.template import TemplateDoesNotExist
from django.template.loader import BaseLoader

from magneto import cache_key
from models import Template


class CachedTemplateLoader(BaseLoader):
    is_usable = True
    timeout = 1800

    def load_template_source(self, template_name, template_dirs=None):
        if template_name == "":
            template_name = "index"

        hash = cache_key(template_name)
        content = cache.get(hash)

        if content == "miss":
            raise TemplateDoesNotExist(template_name)

        if content:
            return (content, template_name)

        try:
            template = Template.on_site.get(name=template_name)
            cache.set(hash, template.content, self.timeout)
            return (template.content, template_name)
        except:
            cache.set(hash, "miss")
            raise TemplateDoesNotExist(template_name)



from distutils.core import setup
import os


AUTHOR = "Preston Timmons"
AUTHOR_EMAIL = "prestontimmons@gmail.com"
PACKAGE_NAME = "django-magneto"
MODULE_NAME = "magneto"
VERSION = "0.9.0"
DESCRIPTION="A content and template management application for Django."
URL="http://bitbucket.org/prestontimmons/django-magneto/src"


# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir:
    os.chdir(root_dir)

for dirpath, dirnames, filenames in os.walk(MODULE_NAME):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        pkg = dirpath.replace(os.path.sep, '.')
        if os.path.altsep:
            pkg = pkg.replace(os.path.altsep, '.')
        packages.append(pkg)
    elif filenames:
        prefix = dirpath[6:] # Strip module name
        for f in filenames:
            data_files.append(os.path.join(prefix, f))


setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=open("README.rst").read(),
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    package_dir={MODULE_NAME: MODULE_NAME},
    packages=packages,
    package_data={MODULE_NAME: data_files},
    classifiers=[
       'Environment :: Web Environment',
       'Framework :: Django',
       'Intended Audience :: Developers',
       'License :: OSI Approved :: BSD License',
       'Programming Language :: Python',
    ],
)



import hashlib

from django.conf import settings


def cache_key(name):
    """
    Create a cache key based on the template name and the current site id.
    """
    return hashlib.md5("magneto-%s-%s" % (name, settings.SITE_ID)).hexdigest()
